/*
    Copyright (c) 2010, 2011 aizo ag, Zurich, Switzerland

    Author: Michael Tross <mt@aizo.ag>

    This file is part of dS-Assistant.

    dS-Assistant is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    dS-Assistant is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

    NOTE: all APIs, especially the JSON API, offered by this application
          are subject to change without further notice.
*/

#ifndef __HWSTATS_H__
#define __HWSTATS_H__

#include <boost/shared_ptr.hpp>
#include <inttypes.h>
#include <string>
#include <list>
#include <map>

/// \file hwstats.h
/// \brief Hardware statistic classes.

namespace dSA
{

#include "hwstats.inc"

typedef enum
{
    unsignedint,
    signedint,
    dsc11temp,
    zeroxcount,
    rxquality,
    bitquality,
    carrierquality,
    invalid,
} stats_value_t;

typedef struct _stats_field_value
{
    statistic_field_t field;
    uint8_t size;
    stats_value_t type;
} stats_field_value_t;

class StatisticValue
{
public:
    StatisticValue(statistic_field_t field, stats_value_t type, time_t t0, long long int value);
    StatisticValue(statistic_field_t field, stats_value_t type, time_t t0, int errorcode);

    statistic_field_t m_field;
    stats_value_t m_type;
    time_t m_time;
    long long int m_value;
    int m_errorcode;
};

class StatisticBase
{
public:
    StatisticBase(uint16_t version);
    virtual ~StatisticBase();

    uint16_t getVersion() const;

    void pushValue(uint8_t val, int errorcode);
    void pushValue(uint16_t val, int errorcode);
    void setValue(uint8_t offset, uint16_t val, int errorcode);
    void parseValues();

    void getOffsetForType(statistic_field_t field, uint8_t &offset, uint8_t &length);

    boost::shared_ptr<StatisticValue> getValueForType(statistic_field_t field);
    std::string formatValue(boost::shared_ptr<StatisticValue> pval);
    boost::shared_ptr<std::map<statistic_field_t, std::string> > formatValues();

    boost::shared_ptr<std::list<boost::shared_ptr<StatisticValue> > > m_value_map;

protected:
    uint16_t m_version;
    uint8_t m_index;
    uint8_t m_data[256];
    bool m_valid[256];
    int m_error[256];
    time_t m_time[256];

    stats_field_value_t *m_definition_table;
    int m_definition_table_length;

    uint8_t getValue8(uint8_t index) const;
    uint16_t getValue16(uint8_t index) const;
    uint32_t getValue32(uint8_t index) const;
    uint64_t getValue64(uint8_t index) const;

    bool isValid(statistic_field_t index);

    std::string getValueString(const stats_field_value_t* item, int offset);
    long long int getValueInt(const stats_field_value_t *item, int offset);

    void setDescriptionTable(const stats_field_value_t* table, int length);
};

class StatisticDevice : public StatisticBase
{
public:
    StatisticDevice(uint8_t version);

};

class StatisticMeter : public StatisticBase
{
public:
    StatisticMeter(uint16_t version);

};

}
#endif
